<?php

namespace Roots\SageVirtualPage;

use Roots\Acorn\Application;

class VirtualPage
{
    /**
     * The application instance.
     *
     * @var \Roots\Acorn\Application
     */
    protected $app;

    /**
     * Create a new Virtual Page instance.
     *
     * @param  \Roots\Acorn\Application  $app
     * @return void
     */
    public function __construct(Application $app)
    {
        $this->app = $app;

        add_action('init', function () {
            foreach ($this->app['config']['virtual-pages'] as $page) {
                add_rewrite_endpoint($page['slug'], EP_PERMALINK);
            }
        });

        add_filter('template_include', function ($template) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    $this->app['sage.view'] = 'pages.'.$page['slug'];
                    // TODO Set $this->app['sage.data'] from an array of data in the config file
                }
            }

            return $template;
        }, 200);

        add_filter('the_posts', function (array $posts, \WP_Query $query) {
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $query->query_vars) || $query->query_vars['name'] === $page['slug']) {
                    $title = $page['title'];
                    $post = [
                        'ID' => 0,
                        'post_title' => $title,
                        'post_name' => sanitize_title($title),
                        'post_content' => '',
                        'post_excerpt' => $page['excerpt'],
                        'post_parent' => 0,
                        'menu_order' => 0,
                        'post_type' => 'page',
                        'post_status' => 'publish',
                        'comment_status' => 'closed',
                        'ping_status' => 'closed',
                        'comment_count' => 0,
                        'post_password' => '',
                        'to_ping' => '',
                        'pinged' => '',
                        'guid' => home_url($query->getUrl()),
                        'post_date' => current_time('mysql'),
                        'post_date_gmt' => current_time('mysql', 1),
                        'post_author' => 0,
                        'is_virtual' => true,
                        'filter' => 'raw',
                    ];

                    return [
                        new \WP_Post((object) $post),
                    ];
                }
            }

            return $posts;
        }, 10, 2);

        \Roots\add_filters([
                'the_seo_framework_description_output',
                'the_seo_framework_ogdescription_output',
                'the_seo_framework_twitterdescription_output',
            ],
            function ($description) {
                global $wp_query;
                foreach ($this->app['config']['virtual-pages'] as $page) {
                    if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                        return $wp_query->post->post_excerpt;
                    }
                }

                return $description;
            }
        );

        add_filter('the_seo_framework_supported_post_type', function ($supported, $post_type_evaluated) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    return true;
                }
            }

            return $supported;
        }, 10, 2);

        add_filter('the_seo_framework_current_object_id', function ($id) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    return get_option('page_on_front', true); // TSF needs a valid page ID in order to output markup, let's use the home page
                }
            }

            return $id;
        });

        add_filter('the_seo_framework_title_from_generation', function ($title, $args) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    return $wp_query->post->post_title;
                }
            }

            return $title;
        }, 10, 2);


        add_filter('the_seo_framework_rel_canonical_output', function ($url, $id) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    return home_url($page['slug'].'/');
                }
            }

            return $url;
        }, 10, 2);

        // Disable breadcrumb output on virtual pages because TSF doesn't support customizing the canonical URL used in the output
        add_filter('the_seo_framework_json_breadcrumb_output', function ($output) {
            global $wp_query;
            foreach ($this->app['config']['virtual-pages'] as $page) {
                if (array_key_exists($page['slug'], $wp_query->query_vars) || array_key_exists('name', $wp_query->query_vars) && $wp_query->query_vars['name'] === $page['slug']) {
                    return false;
                }
            }

            return $output;
        });
    }
}
